package com.persagy.dmp.aspect;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.IntegratedResponseResult;
import com.persagy.dmp.common.VideoResult;
import com.persagy.dmp.constant.ResponseCode;
import com.persagy.dmp.constant.ResponseMsg;
import com.persagy.dmp.enumeration.EnumResult;
import com.persagy.dmp.utils.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;


/**
 * @author LuoGuangyi
 * @version 1.0.0
 * @Description 处理feign调用接口，接口返回值异常的情况
 * feign调用只要返回结果有异常，那么向上抛出异常，特殊不需要抛出异常的地方，需要自己try处理
 * @ClassName FeignClientAspect.java
 * @createTime 2020/12/03 14:11
 * @Since
 */
@Slf4j
@Aspect
@Component
public class FeignClientAspect {
    @Pointcut("execution(* com.persagy.*.client..*.*(..))")
    public void feignPointCut() {
    }

    @Before("feignPointCut()")
    public void before() {
    }
    @Around("feignPointCut()")
    public Object handlerPageAndSize(ProceedingJoinPoint pjp) throws Throwable {
        //中台page没size默认50，改成不限制
        //首先获取方法名称列表
        MethodSignature msg = (MethodSignature)pjp.getSignature();
        String[] paramName = msg.getParameterNames();
        List<String> paramNameList = Arrays.asList(paramName);
        //获取传入的参数
        Object[] args = pjp.getArgs();
        if (ObjectUtil.isNotNull(args) && args.length > 0) {
            for (int i = 0; i < args.length; i++) {
                Object arg = args[i];
                if(isJSONObject(arg)) {
                    JSONObject argJson = getJSONObject(arg);
                    if(getTrueNumber(argJson.containsKey("page"),argJson.containsKey("size")) == 1) {
                        argJson.remove("page");
                        argJson.remove("size");
                        args[i] = argJson;
                    }
                }
            }

//            //返回参数位置
//            Integer pos = paramNameList.indexOf("userId");
//            Long paramValue = (Long)args[pos];
//            args[pos] =paramValue;
        }
        //返回
        Object retVal = pjp.proceed(args);
        return retVal;
    }

    private static int getTrueNumber(boolean... args) {
        int count = 0;
        for (boolean element : args) {
            if (element) {
                count++;
            }
        }
        return count;
    }


    public static boolean isJSONObject(Object content) {
        try {
            if(content == null) {
                return false;
            }
            if(content instanceof String) {
                Object jsobj = JSONObject.parse((String) content);
                return jsobj instanceof JSONObject;
            }else {
                Object jsobj = JSONObject.parse(JSON.toJSONString(content));
                return jsobj instanceof JSONObject;
            }
        } catch (Exception e) {
            return false;
        }
    }

    public static JSONObject getJSONObject(Object content) {
        try {
            if(content == null) {
                return new JSONObject();
            }
            if(content instanceof String) {
                return JSONObject.parseObject((String) content);
            }else {
                return JSONObject.parseObject(JSON.toJSONString(content));
            }
        } catch (Exception e) {
            return new JSONObject();
        }
    }
    @AfterReturning(returning = "res", pointcut = "feignPointCut()")
    public void after(JoinPoint jp, Object res) throws Exception {
        String methodName = jp.getSignature().getName();
        if (res instanceof DmpResult) {
            if (DmpResult.FAILURE.equals(((DmpResult) res).getResult())) {
                log.error("the method [{}] 异常，[错误信息]:{},[请求参数]:{}", methodName, ((DmpResult) res).getMessage(), JSONObject.toJSONString(jp.getArgs()));
                throw new RuntimeException(((DmpResult) res).getMessage());
            }
            return;
        }
        if (res instanceof IntegratedResponseResult) {
            if (!ResponseCode.A00000.getCode().equals(((IntegratedResponseResult) res).getRespCode())) {
                log.error("the method [{}] 异常，[错误信息]:{},[请求参数]:{}", methodName, ((IntegratedResponseResult) res).getRespMsg(), JSONObject.toJSONString(jp.getArgs()));
                throw new RuntimeException(((IntegratedResponseResult) res).getRespMsg());
            }
            return;
        }
        if (res instanceof ResponseMsg) {
            if (EnumResult.success.getValue().equals(((ResponseMsg) res).getResult())) {
                log.error("the method [{}] 异常，[错误信息]:{},[请求参数]:{}", methodName, ((ResponseMsg) res).getResultMsg(), JSONObject.toJSONString(jp.getArgs()));
                throw new RuntimeException(((ResponseMsg) res).getResultMsg());
            }
            return;
        }
        if (res instanceof VideoResult) {
            if (VideoResult.FAILURE.equals(((VideoResult) res).getResult())) {
                log.error("the method [{}] 异常，[错误信息]:{},[请求参数]:{}", methodName, ((DmpResult) res).getMessage(), JSONObject.toJSONString(jp.getArgs()));
                throw new RuntimeException(((VideoResult) res).getReason());
            }
            return;
        }
    }
}
