package com.persagy.dmp.cache;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Joiner;
import com.persagy.dmp.constant.RedisConstant;
import com.persagy.dmp.service.ObjectService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @description: 对象缓存
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/30 16:47
 * @version: V1.0
 **/
@Component
@Slf4j
public class ObjectCache {

    private static final Joiner joiner = Joiner.on(":").skipNulls();

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private ObjectService objectService;

    public JSONArray getObjectByClassCode(String groupCode, String projectId, String classCode) {
        try {
            String key = joiner.join(RedisConstant.OBJECT_PREFIX, groupCode, projectId, classCode);
            Set<String> keys = null;
            if (classCode.length() == 4) {
                //classCode为系统编码是，模式匹配会匹配到系统下设备类的对象，故模糊匹配机上Sy前缀
                keys = redisTemplate.keys(key + "Sy*");
            } else {
                keys = redisTemplate.keys(key + "*");
            }
            Object object = redisTemplate.opsForValue().multiGet(keys);
            return JSONArray.parseArray(object.toString());
        } catch (Exception e) {
            log.error("根据对象类型获取对象列表缓存异常", e);
        }
        return null;
    }

    public JSONObject getObjectById(String groupCode, String projectId, String objId) {
        try {
            String classCode = objectService.queryClassCode(groupCode, projectId, objId);
            String key = joiner.join(RedisConstant.OBJECT_PREFIX, groupCode, projectId, classCode, objId);
            return (JSONObject) redisTemplate.opsForValue().get(key);
        } catch (Exception e) {
            log.error("根据id获取对象缓存异常", e);
        }
        return null;
    }

    public JSONArray getObjectByIds(String groupCode, String projectId, List<String> objIds) {
        JSONArray objArr = new JSONArray();
        try {
            if (!CollectionUtils.isEmpty(objIds)) {
                Set<String> keys = new HashSet<>();
                objIds.forEach(o -> {
                    String classCode = null;
                    try {
                        classCode = objectService.queryClassCode(groupCode, projectId, o);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    keys.add(joiner.join(RedisConstant.OBJECT_PREFIX, groupCode, projectId, classCode, o));
                });
                List<Object> objects = redisTemplate.opsForValue().multiGet(keys);
                if (!CollectionUtils.isEmpty(objects)) {
                    objects.stream().filter(item-> ObjectUtil.isNotEmpty(item)).forEach(o -> {
                        objArr.add(JSONObject.parseObject(o.toString()));
                    });
                }
            }
        } catch (Exception e) {
            log.error("根据ids获取对象列表缓存异常", e);
        }
        return objArr;
    }


    public void putObject(String groupCode, String projectId, String classCode, JSONArray objects) {
        try {
            Random random = new Random();
            if (!CollectionUtils.isEmpty(objects)) {
                Map<String, JSONObject> map = new HashMap<>();
                objects.forEach(o -> {
                    JSONObject jsonObject = (JSONObject) o;
                    String key = joiner.join(RedisConstant.OBJECT_PREFIX, groupCode, projectId, classCode, jsonObject.getString("id"));
                    map.put(key, jsonObject);
                });
                redisTemplate.opsForValue().multiSet(map);
                objects.forEach(
                        o -> {
                            JSONObject jsonObject = (JSONObject) o;
                            String key = joiner.join(RedisConstant.OBJECT_PREFIX, groupCode, projectId, classCode, jsonObject.getString("id"));
                            //随机设定60到120分钟的过期时间
                            redisTemplate.expire(key, random(random, 120, 60), TimeUnit.MINUTES);
                        });
            }
        } catch (Exception e) {
            log.error("缓存对象列表异常", e);
        }
    }

    public void putObject(String groupCode, String projectId, JSONObject object) {
        try {
            String classCode = object.getString("classCode");
            String key = joiner.join(RedisConstant.OBJECT_PREFIX,groupCode, projectId, classCode, object.getString("id"));
            redisTemplate.opsForValue().set(key, object, random(new Random(), 120, 60), TimeUnit.MINUTES);
        } catch (Exception e) {
            log.error("缓存对象详情异常", e);
        }
    }

    public void putObject(String groupCode, String projectId, JSONArray objects) {
        try {
            if (!CollectionUtils.isEmpty(objects)) {
                objects.forEach(
                        o -> {
                            JSONObject jsonObject = (JSONObject) o;
                            putObject(groupCode, projectId, jsonObject);
                        }
                );
            }
        } catch (Exception e) {
            log.error("缓存对象列表异常", e);
        }
    }


    public void putClassCode(String groupCode, String projectId, Map<String, String> objId2ClassCode) {
        try {
            String key = joiner.join(RedisConstant.CLASS_CODE_REL_PREFIX, groupCode, projectId);
            redisTemplate.opsForHash().putAll(key, objId2ClassCode);
        } catch (Exception e) {
            log.error("缓存对象与对象类型关系异常", e);
        }
    }

    public Map<String, String> getClassCode(String groupCode, String projectId, List<String> objIds) {
        Map<String, String> objId2ClassCode = new HashMap<>();
        try {
            if (CollectionUtils.isEmpty(objIds)) {
                return objId2ClassCode;
            }
            String key = joiner.join(RedisConstant.CLASS_CODE_REL_PREFIX, groupCode, projectId);
            objIds.forEach(
                    o -> {
                        Object object = redisTemplate.opsForHash().get(key, o);
                        if (object != null) {
                            objId2ClassCode.put(o, (String) object);
                        }
                    }
            );

        } catch (Exception e) {
            log.error("获取对象与对象类型关系缓存异常", e);
        }
        return objId2ClassCode;
    }

    private int random(Random random, int max, int min) {
        return random.nextInt(max - min + 1) + max;
    }
}
