package com.persagy.dmp.dao;

import java.util.List;

import javax.validation.constraints.NotNull;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.dmp.common.PageResult;
import com.persagy.dmp.dto.ObjRelationDTO;
import com.persagy.dmp.dto.RelationFromDTO;
import com.persagy.dmp.dto.RelationToDTO;
import com.persagy.dmp.dto.relation.BatchRelationCriteria;
import com.persagy.dmp.dto.relation.CreateRelation;
import com.persagy.dmp.dto.relation.RelationCriteria;

import lombok.NonNull;

/**
 * @description: 物理世界对象、关系实例
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/6/3 16:56
 * @version: V1.0
 */
public interface InstanceDao {

    /**
     * @description: 查询关系对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromId from对象id
     * @param: toClassCode to对象类型
     * @param: withColumns 附带对象信息点
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 15:56
     * @version: V1.0
     */
    JSONArray queryRelationObjectByFromId(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, @NotNull String fromId, List<String> toClassCode, List<String> withColumns) throws Exception;

    /**
     * 获取系统下指定设备
     * 
     * @param groupCode
     * @param projectId
     * @param graphCode
     * @param relCode
     * @param fromId
     * @param ibmsSceneCode
     * @param ibmsClassCode
     * @param withColumns
     * @return
     * @throws Exception
     * @date 2021年12月24日 下午4:03:05
     */
    JSONArray queryRelationObjectByFromId(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, @NotNull String fromId, 
    		String ibmsSceneCode, String ibmsClassCode, List<String> withColumns) throws Exception;

    /**
     * @description: 查询关系对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromIds from对象id列表
     * @param: toClassCode to对象类型
     * @param: withColumns 附带对象信息点
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 15:56
     * @version: V1.0
     */
    JSONArray queryRelationObjectByFromIds(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, @NotNull List<String> fromIds, List<String> toClassCode, List<String> withColumns) throws Exception;

    /**
     * @description: 查询关系对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromClassCode from对象类型
     * @param: toId to对象id
     * @param: withColumns 附带对象信息点
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/29 11:44
     * @version: V1.0
     */
    JSONArray queryRelationObjectByToId(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, List<String> fromClassCode, @NotNull String toId, List<String> withColumns) throws Exception;

    /**
     * @description: 查询对象关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: relCode 边类型
     * @param: fromClassCode from对象类型
     * @param: toId to对象id
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:52
     * @version: V1.0
     */
    JSONArray queryRelationObjectByToId(String groupCode, String projectId, String graphCode, String relCode, String toId, String ibmsSceneCode, String ibmsClassCode) throws Exception;

    /**
     * @description: 查询关系对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromClassCode from对象类型
     * @param: toIds to对象id列表
     * @param: withColumns 附带对象信息点
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/29 11:44
     * @version: V1.0
     */
    JSONArray queryRelationObjectByToIds(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, List<String> fromClassCode, @NotNull List<String> toIds, List<String> withColumns) throws Exception;

    /**
     * @description: 查询关系对象（fromIds和toIds只能传一个）
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromIds from对象id
     * @param: toIds to对象id列表
     * @param: objectCriteria 查询对象条件
     * @param: withColumns 查询对象信息点
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/30 10:40
     * @version: V1.0
     */
    JSONArray queryRelationObject(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, List<String> fromIds, List<String> toIds, JSONObject objectCriteria, List<String> withColumns) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 查询关系
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromId from对象id
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDto>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:22
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByFromId(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, @NotNull String fromId) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 查询关系
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromIds from对象id列表
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDto>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:22
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByFromIds(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, @NotNull List<String> fromIds) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: toId to对象id
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDto>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:21
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByToId(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, @NotNull String toId) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: toIds to对象id列表
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDto>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:20
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByToIds(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, @NotNull List<String> toIds) throws Exception;

    /**
     * @description: 查询关系(fromIds和toIds只能传一个)
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromIds from对象id列表
     * @param: toIds to对象id列表
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDto>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:19
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelation(String groupCode, @NotNull String projectId, @NotNull String graphCode, @NotNull String relCode, List<String> fromIds, List<String> toIds) throws Exception;

    /**
     * @description: 创建关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: createRelationList 创建关系
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean createRelation(String groupCode, @NotNull String projectId, @NotNull CreateRelation createRelation);

    /**
     * @description: 批量创建关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: createRelationList 创建关系
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean createRelation(String groupCode, @NotNull String projectId, @NotNull List<CreateRelation> createRelationList);

    /**
     * @description: 批量删除关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: criteria 删除关系条件
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean batchDeleteRelation(String groupCode, @NotNull String projectId, @NotNull BatchRelationCriteria criteria);

    /**
     * @description: 删除关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: criteria 删除关系条件
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean deleteRelation(String groupCode, @NotNull String projectId, @NotNull RelationCriteria criteria);

    /**
     * @description: 批量创建对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: classCode 对象类型
     * @param: objectInfo 创建对象信息点
     * @return: void
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:17
     * @version: V1.0
     */
    Boolean createObject(String groupCode, String projectId, @NonNull String classCode, JSONObject objectInfo);

    /**
     * @description: 批量创建对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectArr 创建对象列表
     * @return: void
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:17
     * @version: V1.0
     */
    Boolean createObject(String groupCode, String projectId, JSONArray objectArr);

    /**
     * @description: 批量更新对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 更新对象id
     * @param: objectArr 更新对象信息点
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:16
     * @version: V1.0
     */
    Boolean updateObject(String groupCode, String projectId, String objectId, JSONObject objectInfo);

    /**
     * @description: 批量更新对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectArr 更新对象列表
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:16
     * @version: V1.0
     */
    Boolean updateObject(String groupCode, String projectId, JSONArray objectArr);

    /**
     * @description: 删除对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 删除对象id
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:15
     * @version: V1.0
     */
    Boolean deleteObject(String groupCode, String projectId, String objectId);

    /**
     * @description: 批量删除对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 删除对象id列表
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:15
     * @version: V1.0
     */
    Boolean deleteObject(String groupCode, String projectId, List<String> objectIdList);

    /**
     * @description: 获取对象信息点
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @return: com.alibaba.fastjson.JSONObject
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 17:06
     * @version: V1.0
     */
    JSONObject queryObjectInfo(String groupCode, String projectId, String objectId) throws Exception;
    
    /**
     * 获取对象信息
     * 
     * @param groupCode
     * @param projectId
     * @param ibmsSceneCode
     * @param ibmsClassCode
     * @param objectId
     * @return
     * @throws Exception
     * @date 2021年12月24日 上午10:48:11
     */
    JSONObject queryObjectInfo(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, String objectId) throws Exception;

    /***
     * @description: 根据classCode获取对象列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: classCode 设备类型编码
     * @return: JSONArray
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 11:27 2020/6/23
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, String projectId, String classCode,List<String> withColumns) throws Exception;

    PageResult<JSONArray> queryObjectList(String groupCode, @NotNull String projectId, @NotNull String ibmsSceneCode, @NotNull String ibmsClassCode, List<String> withColumns) throws Exception;

    /***
     * @description: 根据对象id获取项目列表
     * @param: groupCode 集团编码
     * @param: projectId 建筑id
     * @param: objectIdList 对象列表
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    JSONArray queryObjectList(String groupCode, String projectId, List<String> objectIdList) throws Exception;

    /***
     * @description: 根据对象id获取项目列表
     * @param: groupCode 集团编码
     * @param: projectId 建筑id
     * @param: criteria 查询条件
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, @NotNull String projectId, @NotNull JacksonCriteria criteria) throws Exception;

    /***
     * @description: 根据对象id获取项目列表
     * @param: groupCode 集团编码
     * @param: projectId 建筑id
     * @param: criteria 查询条件
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, @NotNull String projectId, @NotNull JSONObject criteria) throws Exception;

    /***
     * @description: 根据对象id获取项目列表
     * @param: groupCode 集团编码
     * @param: projectId 建筑id
     * @param: criteria 对象查询条件
     * @param: relationFromDTO from关系查询条件
     * @param: relationToDTO to关系查询条件
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, @NotNull String projectId, @NotNull JacksonCriteria criteria, RelationFromDTO relationFromDTO, RelationToDTO relationToDTO) throws Exception;
}
