package com.persagy.dmp.dao.impl;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.client.AlarmNewServerClient;
import com.persagy.dmp.client.GroupServerClient;
import com.persagy.dmp.client.RwdServerClient;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.url.ClassUrParam;
import com.persagy.dmp.common.url.FuncIdUrParam;
import com.persagy.dmp.dao.DefDao;
import com.persagy.dmp.dto.FuncIdQueryDTO;
import com.persagy.dmp.dto.InfoPointDictDTO;
import com.persagy.dmp.dto.QueryFuncId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 数据字典类接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/6/3 16:55
 * @version: V1.0
 */
@Component
public class DefDaoImpl implements DefDao {

    private static final Logger logger = LoggerFactory.getLogger(DefDaoImpl.class);

    @Autowired
    private RwdServerClient rwdServerClient;

    @Autowired
    private AlarmNewServerClient alarmNewServerClient;

    /***
     * @description: 获取数据字典平台级信息点属性信息
     * @param: projectId 项目id
     * @param: classCode 类型编码
     * @return: java.util.List<com.persagy.huiyun.dto.InfoPointDictDTO>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 17:14
     * @version: V1.0
     */
    @Override
    @Cacheable(value = "dataDict")
    public List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, String classCode) throws Exception {
        FuncIdQueryDTO funcIdQueryDto = new FuncIdQueryDTO();
        funcIdQueryDto.setClassCode(classCode);
        return this.queryInfoPointDictList(groupCode, projectId, funcIdQueryDto);
    }

    /**
     * @description: 获取数据字典信息点属性信息
     * @param: projectId 项目id
     * @param: funcIdQueryDto 查询信息点参数类
     * @return: java.util.List<com.persagy.common.dto.InfoPointDictDTO>
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/29 10:56
     * @version: V1.0
     */
    @Override
    @Cacheable(value = "dataDict")
    public List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, FuncIdQueryDTO funcIdQueryDto) throws Exception {
        FuncIdUrParam funcIdUrParam = FuncIdUrParam.builder().groupCode(groupCode).projectId(projectId).build();
        QueryFuncId queryFuncId = QueryFuncId.builder().criteria(funcIdQueryDto).build();
        DmpResult<JSONArray> dmpResult =  rwdServerClient.queryFuncId(funcIdUrParam, queryFuncId);
        if (dmpResult != null && DmpResult.SUCCESS.equalsIgnoreCase(dmpResult.getResult())) {
            JSONArray dataArray = dmpResult.getData();
            if (!CollectionUtils.isEmpty(dataArray)) {
                List<InfoPointDictDTO> infoPointDictDTOList = dataArray.toJavaList(InfoPointDictDTO.class);
                //信息点别名覆盖信息点名称
                infoPointDictDTOList.forEach(o -> o.setName(o.getAliasName()));
                return infoPointDictDTOList;
            }
        }
        return new ArrayList<>();
    }

    @Override
    public JSONArray queryClassInfoList(String groupCode, @NotNull String projectId, Boolean subFlag, @NotNull JacksonCriteria criteria) throws Exception {
        ClassUrParam classUrParam = ClassUrParam.builder().groupCode(groupCode).projectId(projectId).subFlag(subFlag).build();
        JSONObject queryParam = JSONObject.parseObject(JacksonMapper.toFormatJson(criteria));
        DmpResult<JSONArray> dmpResult = rwdServerClient.queryClass(classUrParam, queryParam);
        if (dmpResult != null && DmpResult.SUCCESS.equalsIgnoreCase(dmpResult.getResult())) {
            JSONArray dataArray = dmpResult.getData();
            return dataArray;
        }
        return new JSONArray();
    }
}
