package com.persagy.dmp.dao.impl;

import com.alibaba.fastjson.JSONArray;
import com.persagy.dmp.client.RwdServerClient;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.url.IOTUrlParam;
import com.persagy.dmp.dao.IOTDao;
import com.persagy.dmp.dto.*;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * @description: iot数据接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/15 11:58
 * @version: V1.0
 **/
@Component
public class IOTDaoImpl implements IOTDao {

    @Autowired
    private RwdServerClient dmpServerClient;

    /**
     * @description: 查询实时数据
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCode 信息点编码
     * @return: com.persagy.common.dto.CurrentDataDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    @Override
    public CurrentDataDTO queryCurrentData(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull String infoCode) throws Exception {
        List<CurrentDataDTO> currentDataDTOList = queryCurrentDataList(groupCode, projectId, objectId, Arrays.asList(infoCode));
        if (!CollectionUtils.isEmpty(currentDataDTOList)) {
            return currentDataDTOList.get(0);
        }
        return new CurrentDataDTO();
    }

    /**
     * @description: 查询实时数据
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @param: infoCode 信息点编码
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    @Override
    public List<CurrentDataDTO> queryCurrentDataList(String groupCode, @NotNull String projectId, @NotNull List<String> objectIdList, @NotNull String infoCode) throws Exception {
        List<DataQueryDTO> dataQueryDTOList = new ArrayList<>();
        for (String objectId : objectIdList) {
            DataQueryDTO dataQueryDto = new DataQueryDTO();
            dataQueryDto.setObjectId(objectId);
            dataQueryDto.setInfoCode(infoCode);
            dataQueryDTOList.add(dataQueryDto);
        }
        return queryCurrentDataList(groupCode, projectId, dataQueryDTOList);
    }

    /**
     * @description: 查询实时数据
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCodeList 信息点编码列表
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:11
     * @version: V1.0
     */
    @Override
    public List<CurrentDataDTO> queryCurrentDataList(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull List<String> infoCodeList) throws Exception {
        List<DataQueryDTO> dataQueryDTOList = new ArrayList<>();
        for (String infoCode : infoCodeList) {
            DataQueryDTO dataQueryDto = new DataQueryDTO();
            dataQueryDto.setObjectId(objectId);
            dataQueryDto.setInfoCode(infoCode);
            dataQueryDTOList.add(dataQueryDto);
        }
        return this.queryCurrentDataList(groupCode, projectId, dataQueryDTOList);
    }

    /**
     * @description: 查询实时数据
     * @param: projectId 项目id
     * @param: dataQueryDtoList 实时数据参数（对象id、信息点）列表
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:11
     * @version: V1.0
     */
    private List<CurrentDataDTO> queryIotCurrentDataList(String groupCode, @NonNull String projectId, @NotNull List<DataQueryDTO> dataQueryDTOList) throws Exception {
        //构建请求url上参数
        IOTUrlParam iotUrlParam = IOTUrlParam.builder().groupCode(groupCode).projectId(projectId).build();
        DmpResult<JSONArray> dmpResult = dmpServerClient.queryIotCurrentData(iotUrlParam, dataQueryDTOList);
        if (dmpResult != null && DmpResult.SUCCESS.equalsIgnoreCase(dmpResult.getResult())) {
            JSONArray dataArray = dmpResult.getData();
            if (!CollectionUtils.isEmpty(dataArray)) {
                return dataArray.toJavaList(CurrentDataDTO.class);
            }
        }
        return new ArrayList<>();
    }

    @Override
    public List<CurrentDataDTO> queryCurrentDataList(String groupCode, @NonNull String projectId, @NotNull List<DataQueryDTO> dataQueryDTOList) throws Exception {

        if (CollectionUtils.isEmpty(dataQueryDTOList)) {
            return new ArrayList<>();
        }

        int threadCounts = dataQueryDTOList.size() / 100;
        if (threadCounts == 0) {
            threadCounts = 1;
        } else if (threadCounts > 32) {
            threadCounts = 32;
        }

        ExecutorService exec = Executors.newFixedThreadPool(threadCounts);
        List<Callable<List<CurrentDataDTO>>> callList = new ArrayList<Callable<List<CurrentDataDTO>>>();

        //平均分割List
        int len = dataQueryDTOList.size() / threadCounts;
        //List中的数量没有线程数多（很少存在）
        if (len == 0) {
            //采用一个线程处理List中的一个元素
            threadCounts = dataQueryDTOList.size();
            //重新平均分割List
            len = dataQueryDTOList.size() / threadCounts;
        }

        for (int i = 0; i < threadCounts; i++) {
            final List<DataQueryDTO> subList;
            if (i == threadCounts - 1) {
                subList = dataQueryDTOList.subList(i * len, dataQueryDTOList.size());
            } else {
                subList = dataQueryDTOList.subList(i * len, len * (i + 1) > dataQueryDTOList.size() ? dataQueryDTOList.size() : len * (i + 1));
            }
            //采用匿名内部类实现
            callList.add(() -> IOTDaoImpl.this.queryIotCurrentDataList(groupCode, projectId, subList));
        }

        List<Future<List<CurrentDataDTO>>> futureList = exec.invokeAll(callList);
        List<CurrentDataDTO> currentDataDTOList = new ArrayList<>();
        for (Future<List<CurrentDataDTO>> future : futureList) {
            List<CurrentDataDTO> subCurrentDataDTOList = future.get();
            if (!CollectionUtils.isEmpty(subCurrentDataDTOList)) {
                currentDataDTOList.addAll(subCurrentDataDTOList);
            }
        }
        exec.shutdown();

        return currentDataDTOList;
    }

    /**
     * @description: 查询历史数据/分精度数据
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCode 信息点编码
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: com.persagy.common.dto.HistoryDataDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:06
     * @version: V1.0
     */
    @Override
    public HistoryDataDTO queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull String infoCode, @NotNull String startTime, String endTime, @NotNull String period) throws Exception {
        List<HistoryDataDTO> historyDataDTOList = queryHistoryDataList(groupCode, projectId, objectId, Arrays.asList(infoCode), startTime, endTime, period);
        if (!CollectionUtils.isEmpty(historyDataDTOList)) {
            return historyDataDTOList.get(0);
        }
        return new HistoryDataDTO();
    }

    /**
     * @description: 查询历史数据/分精度数据
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCodeList 信息点编码列表
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:06
     * @version: V1.0
     */
    @Override
    public List<HistoryDataDTO> queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull List<String> infoCodeList, @NotNull String startTime, String endTime, @NotNull String period) throws Exception {
        List<DataQueryDTO> dataQueryDTOList = new ArrayList<>();
        for (String infoCode : infoCodeList) {
            DataQueryDTO dataQueryDto = new DataQueryDTO();
            dataQueryDto.setObjectId(objectId);
            dataQueryDto.setInfoCode(infoCode);
            dataQueryDTOList.add(dataQueryDto);
        }
        return queryHistoryDataList(groupCode, projectId, dataQueryDTOList, startTime, endTime, period);
    }

    /**
     * @description: 查询历史数据/分精度数据
     * @param: projectId 项目id
     * @param: objectIdList 对象id列表
     * @param: infoCode 信息点编码
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:05
     * @version: V1.0
     */
    @Override
    public List<HistoryDataDTO> queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull List<String> objectIdList, @NotNull String infoCode, @NotNull String startTime, String endTime, String period) throws Exception {
        List<DataQueryDTO> dataQueryDTOList = new ArrayList<>();
        for (String objectId : objectIdList) {
            DataQueryDTO dataQueryDto = new DataQueryDTO();
            dataQueryDto.setObjectId(objectId);
            dataQueryDto.setInfoCode(infoCode);
            dataQueryDTOList.add(dataQueryDto);
        }
        return queryHistoryDataList(groupCode, projectId, dataQueryDTOList, startTime, endTime, period);
    }

    /**
     * @description: 查询历史数据/分精度数据
     * @param: projectId 项目id
     * @param: dataQueryDtoList 查询历史数据的对象id、信息点信息点编码类列表
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔 1min、5min、15min、30min、1h、2h、1d
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:04
     * @version: V1.0
     */
    @Override
    public List<HistoryDataDTO> queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull List<DataQueryDTO> dataQueryDTOList, @NotNull String startTime, String endTime, @NotNull String period) throws Exception {

        String newPeriod = period;
        if ("30min".equals(period)) {
            newPeriod = "15min";
        } else if ("2h".equals(period)) {
            newPeriod = "1h";
        }
        //查询历史数据
        List<HistoryDataDTO> historyDataDTOList = queryHistoryDataList2(groupCode, projectId, dataQueryDTOList, startTime, endTime, newPeriod);

        if (CollectionUtils.isEmpty(historyDataDTOList)) {
            return new ArrayList<>();
        }
        if ("30min".equals(period) || "2h".equals(period)) {
            for (HistoryDataDTO historyDataDTO : historyDataDTOList) {
                List<DataTimeDTO> dataTimeDTOList = historyDataDTO.getData();
                if (CollectionUtils.isEmpty(dataTimeDTOList)) {
                    continue;
                }
                List<DataTimeDTO> newDataTimeDTOList = new ArrayList<>();
                for (int i = 0; i < dataTimeDTOList.size(); i = i + 2) {
                    newDataTimeDTOList.add(dataTimeDTOList.get(i));
                }
                historyDataDTO.setData(newDataTimeDTOList);
            }

        }
        return historyDataDTOList;
    }


    /**
     * @description: 查询历史数据/分精度数据
     * @param: projectId 项目id
     * @param: dataQueryDtoList 查询历史数据的对象id、信息点信息点编码类列表
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔 1min、5min、15min、1h、1d
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:04
     * @version: V1.0
     */
    private List<HistoryDataDTO> queryHistoryDataList2(String groupCode, @NonNull String projectId, @NotNull List<DataQueryDTO> dataQueryDTOList, @NotNull String startTime, String endTime, @NotNull String period) throws Exception {
        //构建请求url上参数
        IOTUrlParam iotUrlParam = IOTUrlParam.builder().groupCode(groupCode).projectId(projectId).build();

        //构建post参数
        HistoryDataQuery historyDataQuery = HistoryDataQuery.builder().startTime(startTime).endTime(endTime)
                .period(period).params(dataQueryDTOList).build();
        DmpResult<JSONArray> dmpResult = dmpServerClient.queryIotHistoryData(iotUrlParam, historyDataQuery);
        if (dmpResult != null && DmpResult.SUCCESS.equalsIgnoreCase(dmpResult.getResult())) {
            JSONArray dataArray = dmpResult.getData();
            if (!CollectionUtils.isEmpty(dataArray)) {
                return dataArray.toJavaList(HistoryDataDTO.class);
            }
        }
        return new ArrayList<>();
    }
}
