package com.persagy.dmp.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.dmp.common.PageResult;
import com.persagy.dmp.dto.ObjectInfoDTO;
import com.persagy.dmp.dto.RelationFromDTO;
import com.persagy.dmp.dto.RelationToDTO;
import lombok.NonNull;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @description: 对象接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:10
 * @version: V1.0
 **/
public interface ObjectService {

    /**
     * @description: 批量创建对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: classCode 对象类型
     * @param: objectInfo 创建对象信息点
     * @return: void
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:17
     * @version: V1.0
     */
    Boolean createObject(String groupCode, String projectId, @NonNull String classCode, JSONObject objectInfo);

    /**
     * @description: 批量创建对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectArr 创建对象列表
     * @return: void
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:17
     * @version: V1.0
     */
    Boolean createObject(String groupCode, String projectId, JSONArray objectArr);

    /**
     * @description: 批量更新对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 更新对象id
     * @param: objectArr 更新对象信息点
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:16
     * @version: V1.0
     */
    Boolean updateObject(String groupCode, String projectId, String objectId, JSONObject objectInfo);

    /**
     * @description: 批量更新对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectArr 更新对象列表
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:16
     * @version: V1.0
     */
    Boolean updateObject(String groupCode, String projectId, JSONArray objectArr);

    /**
     * @description: 删除对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 删除对象id
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:15
     * @version: V1.0
     */
    Boolean deleteObject(String groupCode, String projectId, String objectId);

    /**
     * @description: 批量删除对象
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 删除对象id列表
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:15
     * @version: V1.0
     */
    Boolean deleteObject(String groupCode, String projectId, List<String> objectIdList);

    /**
     * @description: 查询对象类型编码
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objId 对象id
     * @return: java.lang.String
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/4 10:40
     * @version: V1.0
     */
    String queryClassCode(String groupCode, String projectId, String objId) throws Exception;

    /**
     * @description: 查询对象类型编码
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objIds 对象id列表
     * @return: java.util.Map<java.lang.String, java.lang.String>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/4 10:40
     * @version: V1.0
     */
    Map<String, String> queryClassCode(String groupCode, String projectId, List<String> objIds) throws Exception;

    /**
     * @description: 获取对象信息点
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @return: java.util.Map<java.lang.String, java.lang.Object>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 17:06
     * @version: V1.0
     */
    JSONObject queryObjectInfo(String groupCode, String projectId, String objectId) throws Exception;
    
    /**
     * @description: 获取对象信息点
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @return: java.util.Map<java.lang.String, java.lang.Object>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 17:06
     * @version: V1.0
     */
    JSONObject queryObjectInfoNoCache(String groupCode, String projectId, String objectId) throws Exception;

    /**
     * 获取单设备信息
     * 
     * @param groupCode
     * @param projectId
     * @param ibmsSceneCode
     * @param ibmsClassCode
     * @param objectId
     * @return
     * @throws Exception
     * @date 2021年12月24日 上午10:46:07
     */
    JSONObject queryObjectInfo(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, String objectId) throws Exception;

    /***
     * @description: 根据classCode获取对象列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: classCode 设备类型编码
     * @return: com.persagy.dmp.config.Page<com.alibaba.fastjson.JSONArray>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 11:27 2020/6/23
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, String projectId, String classCode) throws Exception;

    /***
     * @description: 根据classCode获取对象列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: classCode 设备类型编码
     * @return: com.persagy.dmp.config.Page<com.alibaba.fastjson.JSONArray>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 11:27 2020/6/23
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, String projectId, String classCode, List<String> withColumns) throws Exception;

    PageResult<JSONArray> queryObjectList(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, List<String> withColumns) throws Exception;

    /***
     * @description: 根据对象id获取对象列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    JSONArray queryObjectList(String groupCode, String projectId, List<String> objectIdList) throws Exception;

    /**
     * @description: 根据对象id获取对象Map
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @return: java.util.Map<java.lang.String, com.persagy.dmp.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/16 16:51
     * @version: V1.0
     */
    Map<String, ObjectInfoDTO> queryObjectMap(String groupCode, String projectId, List<String> objectIdList) throws Exception;

    /***
     * @description: 根据对象id获取项目列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: criteria 查询条件
     * @return: com.persagy.dmp.config.Page<com.alibaba.fastjson.JSONArray>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, String projectId, JacksonCriteria criteria) throws Exception;

    /***
     * @description: 根据对象id获取项目列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: criteria 查询条件
     * @return: com.persagy.dmp.config.Page<com.alibaba.fastjson.JSONArray>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, String projectId, JSONObject criteria) throws Exception;

    /***
     * @description: 根据对象id获取项目列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: criteria 对象查询条件
     * @param: relationFromDTO from关系查询条件
     * @param: relationToDTO to关系查询条件
     * @return: com.persagy.dmp.config.Page<com.alibaba.fastjson.JSONArray>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 16:56 2020/7/13
     * @version: V1.0
     */
    PageResult<JSONArray> queryObjectList(String groupCode, String projectId, JacksonCriteria criteria, RelationFromDTO relationFromDTO, RelationToDTO relationToDTO) throws Exception;

    /**
     * @description: 根据设备类型查设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: category 设备类型编码
     * @return: java.util.List<com.persagy.common.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:51
     * @version: V1.0
     */
    List<ObjectInfoDTO> queryEquipByCategory(String groupCode, String projectId, String category) throws Exception;

    /**
     * @description: 查询项目下建筑
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @return: java.util.List<com.persagy.common.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:51
     * @version: V1.0
     */
    List<ObjectInfoDTO> queryBuildByProjectId(String groupCode, String projectId) throws Exception;

    /**
     * @description: 查询建筑下楼层
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: buildId 建筑id
     * @return: java.util.List<com.persagy.common.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:50
     * @version: V1.0
     */
    List<ObjectInfoDTO> queryFloorByBuildId(String groupCode, String projectId, String buildId) throws Exception;
    
    /**
     * 
     * @param groupCode
     * @param projectId
     * @param graphCode
     * @param relCode
     * @param toId
     * @return
     * @throws Exception
     */
    JSONObject queryRelationObjectByToId(String groupCode, String projectId, String graphCode, String relCode, String toId) throws Exception;
    
    /**
     * 
     * @param groupCode
     * @param projectId
     * @param graphCode
     * @param relCode
     * @param fromId
     * @return
     * @throws Exception
     */
    JSONObject queryRelationObjectByFromId(String groupCode, String projectId, String graphCode, String relCode, String fromId) throws Exception;
    
    /**
     * 查询系统所在建筑，返回系统信息
     * 
     * @param groupCode
     * @param projectId
     * @param buildId
     * @param withColumns
     * @return
     * @throws Exception
     * @date 2021年12月21日 下午6:01:35
     */
    JSONArray querySystemArrByBuildId(String groupCode, String projectId, String buildId, List<String> withColumns) throws Exception;
    
    /**
     * 查询系统所在楼层，返回系统信息
     * 
     * @param groupCode
     * @param projectId
     * @param buildId
     * @param withColumns
     * @return
     * @throws Exception
     * @date 2021年12月21日 下午6:01:35
     */
    JSONArray querySystemArrByFloorId(String groupCode, String projectId, String floorId, List<String> withColumns) throws Exception;
    
    /**
     * 获取系统服务于空间对应的系统
     * 
     * @param groupCode
     * @param projectId
     * @param spaceId
     * @param withColumns
     * @return
     * @throws Exception
     * @date 2021年12月24日 上午11:24:33
     */
    JSONArray querySystemArrBySpaceId(String groupCode, String projectId, String spaceId, List<String> withColumns) throws Exception;
    
    /**
     * @description: 查询建筑下楼层
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: floorId 楼层id
     * @param: classCode 空间业务分区
     * @param: roomFuncType 空间功能类型
     * @return: java.util.List<com.persagy.common.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:50
     * @version: V1.0
     */
    List<ObjectInfoDTO> querySpaceByFloorId(String groupCode, String projectId, String floorId, String classCode, String roomFuncType) throws Exception;

    /**
     * @description: 查询空间下设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: spaceId 空间id
     * @param: category 设备类型编码
     * @return: java.util.List<com.persagy.common.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:49
     * @version: V1.0
     */
    List<ObjectInfoDTO> queryEquipBySpaceId(String groupCode, String projectId, String spaceId, String category) throws Exception;

    /**
     * @description: 查询楼层下设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: floorId 楼层id
     * @param: category 设备类型编码
     * @return: java.util.List<com.persagy.common.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    List<ObjectInfoDTO> queryEquipByFloorId(String groupCode, String projectId, String floorId, String category) throws Exception;

    /**
     * @description: 查询系统下设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: systemId 系统id
     * @param: classCodeList 设备类列表
     * @return: java.util.List<com.persagy.common.dto.ObjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    List<ObjectInfoDTO> queryEquipBySystemId(String groupCode, String projectId, String systemId, List<String> classCodeList) throws Exception;

    /**
     * @description: 查询项目下建筑
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:51
     * @version: V1.0
     */
    JSONArray queryBuildArrByProjectId(String groupCode, String projectId, List<String> withColumns) throws Exception;

    /**
     * @description: 查询建筑下楼层
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: buildId 建筑id
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:50
     * @version: V1.0
     */
    JSONArray queryFloorArrByBuildId(String groupCode, String projectId, String buildId, List<String> withColumns) throws Exception;

    /**
     * @description: 查询建筑下楼层
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: floorId 楼层id
     * @param: classCode 空间业务分区
     * @param: roomFuncType 空间功能类型
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:50
     * @version: V1.0
     */
    JSONArray querySpaceArrByFloorId(String groupCode, String projectId, String floorId, String classCode, String roomFuncType, List<String> withColumns) throws Exception;

    /**
     * @description: 查询空间下设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: spaceId 空间id
     * @param: category 设备类型编码
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:49
     * @version: V1.0
     */
    JSONArray queryEquipArrBySpaceId(String groupCode, String projectId, String spaceId, String category, List<String> withColumns) throws Exception;

    /**
     * @description: 查询楼层下设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: floorId 楼层id
     * @param: category 设备类型编码
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    JSONArray queryEquipArrByFloorId(String groupCode, String projectId, String floorId, String category, List<String> withColumns) throws Exception;

    /**
     * @description: 查询系统下设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: systemId 系统id
     * @param: classCodeList 设备类列表
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    JSONArray queryEquipArrBySystemId(String groupCode, String projectId, String systemId, List<String> classCodeList, List<String> withColumns) throws Exception;

    /**
     * 获取系统下指定设备
     * 
     * @param groupCode
     * @param projectId
     * @param systemId
     * @param ibmsSceneCode
     * @param ibmsClassCode
     * @param withColumns
     * @return
     * @throws Exception
     * @date 2021年12月24日 下午4:01:35
     */
    JSONArray queryEquipArrBySystemId(String groupCode, String projectId, String systemId, String ibmsSceneCode, String ibmsClassCode, List<String> withColumns) throws Exception;

    /**
     * @description: 查询建筑下设备
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: buildId 建筑id
     * @param: classCodeList 设备类列表
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    JSONArray queryEquipArrByBuildId(String groupCode, String projectId, String buildId, String category, List<String> withColumns) throws Exception;

    /**
     * @description: 校验 对象 信息值 是否重复
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId    对象id
     * @param: infoCode    信息点
     * @param: value   信息点值
     * @return: java.lang.Boolean
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/4 14:46
     * @version: V1.0
     */
    Boolean verifyIsRepeat(String groupCode, @NonNull String projectId, String objectId, @NonNull String infoCode, @NonNull Object value) throws Exception;


    Map<String,String> queryFloorSequenceID( @NonNull String groupCode, @NonNull String projectId, Set<String> floorIds);
}
