package com.persagy.dmp.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.persagy.dmp.constant.DmpConstant;
import com.persagy.dmp.dto.logicalGrouping.LogicalGroupingDTO;
import com.persagy.dmp.entity.LogicalGrouping;
import com.persagy.dmp.mapper.LogicalGroupingMapper;
import com.persagy.dmp.request.DeleteLogicalGroupingRequest;
import com.persagy.dmp.request.LogicalGroupingListRequest;
import com.persagy.dmp.request.UpsertLogicalGroupingRequest;
import com.persagy.dmp.response.LogicalGroupingListResponse;
import com.persagy.dmp.service.LogicalGroupingServiceV2;
import com.persagy.dmp.service.LogicalObjectServiceV2;
import com.persagy.dmp.service.ObjectService;
import com.persagy.dmp.utils.BeanDtoVoUtils;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/16 17:16
 */
@Service
public class LogicalGroupingServiceV2Impl implements LogicalGroupingServiceV2 {

    @Autowired
    private ObjectService objectService;

    @Autowired
    private LogicalGroupingMapper logicalGroupingMapper;

    @Autowired
    private LogicalObjectServiceV2 logicalObjectServiceV2;

    @Override
    public LogicalGroupingListResponse queryLogicalGroupingList(LogicalGroupingListRequest request) {
        String groupCode = request.getGroupCode();
        String projectId = request.getProjectId();
        String ibmsSceneCode = request.getIbmsSceneCode();
        String ibmsClassCode = request.getIbmsClassCode();

        List<LogicalGroupingDTO> groupings = BeanDtoVoUtils.listVo(
                logicalGroupingMapper.selectList(
                        Wrappers.<LogicalGrouping>lambdaQuery()
                                .eq(LogicalGrouping::getGroupCode, groupCode)
                                .eq(LogicalGrouping::getProjectId, projectId)
                                .eq(LogicalGrouping::getIbmsSceneCode, ibmsSceneCode)
                                .eq(LogicalGrouping::getIbmsClassCode, ibmsClassCode)
                                .eq(LogicalGrouping::getParentId, request.getParentLogicalGroupingId())
                                .orderByAsc(LogicalGrouping::getSequence)),
                LogicalGroupingDTO.class);

        int objNum = logicalObjectServiceV2.countOfAllLogicalObjectEquip(
                groupCode,
                projectId,
                ibmsSceneCode,
                ibmsClassCode);


        return LogicalGroupingListResponse.builder()
                .objNum(objNum)
                .groupings(groupings)
                .build();
    }

    @Override
    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    public void upsertLogicalGrouping(UpsertLogicalGroupingRequest request) {
        // 参数校验
        boolean isUpdate = false;
        String logicalGroupingId = request.getLogicalGroupingId();
        String parentLogicalGroupingId = request.getParentLogicalGroupingId();

        if (StringUtils.isNotEmpty(logicalGroupingId)) {
            LogicalGrouping grouping = logicalGroupingMapper.selectById(logicalGroupingId);
            if (Objects.isNull(grouping)) {
                throw new IllegalArgumentException("逻辑编组不存在");
            }

            // 如果是更新, 给parentLogicalGroupingId赋值, 后面需要它来判断是否名称重复
            if (StringUtils.isEmpty(parentLogicalGroupingId)) {
                parentLogicalGroupingId = grouping.getParentId();
            }

            isUpdate = true;
        }

        final String rootParentId = "0";

        if (!isUpdate) {
            if (StringUtils.isEmpty(parentLogicalGroupingId)) {
                parentLogicalGroupingId = rootParentId;
            }

            if (!rootParentId.equals(parentLogicalGroupingId)) {
                if (Objects.isNull(logicalGroupingMapper.selectById(parentLogicalGroupingId))) {
                    throw new IllegalArgumentException("父逻辑编组不存在");
                }
            }
        }

        if (parentLogicalGroupingId.equals(logicalGroupingId)) {
            throw new IllegalArgumentException("逻辑编组和父逻辑编组不能是同一个");
        }

        String groupCode = request.getGroupCode();
        String projectId = request.getProjectId();
        String ibmsSceneCode = request.getIbmsSceneCode();
        String ibmsClassCode = request.getIbmsClassCode();
        String logicalGroupingName = request.getLogicalGroupingName();

        // 校验编组名称
        if (logicalGroupingMapper.selectCount(
                Wrappers.<LogicalGrouping>lambdaQuery()
                        .eq(LogicalGrouping::getGroupCode, groupCode)
                        .eq(LogicalGrouping::getProjectId, projectId)
                        .eq(LogicalGrouping::getIbmsSceneCode, ibmsSceneCode)
                        .eq(LogicalGrouping::getIbmsClassCode, ibmsClassCode)
                        .eq(LogicalGrouping::getLogicalGroupingName, logicalGroupingName)
                        // 同一个父分组下
                        .eq(LogicalGrouping::getParentId, parentLogicalGroupingId)
                        // 更新操作不和自己的模式名称比较
                        .ne(isUpdate, LogicalGrouping::getLogicalGroupingId, logicalGroupingId)
        ) > 0) {
            throw new IllegalArgumentException("逻辑编组名称重复");
        }

        // 构建实体
        LogicalGrouping entity = new LogicalGrouping();
        entity.setLogicalGroupingId(logicalGroupingId);
        entity.setParentId(parentLogicalGroupingId);

        entity.setGroupCode(groupCode);
        entity.setProjectId(projectId);
        entity.setIbmsSceneCode(ibmsSceneCode);
        entity.setIbmsClassCode(ibmsClassCode);

        entity.setLogicalGroupingName(logicalGroupingName);
        entity.setRemark(request.getRemark());
        entity.setFirstCode(request.getFirstCode());
        entity.setSecondCode(request.getSecondCode());

        String floorId = request.getFloorId();
        JSONObject groupExtra = new JSONObject();

        if (StringUtils.isNotEmpty(floorId)) {
            JSONObject floorObject = objectService.queryObjectInfo(groupCode, projectId, floorId);

            if (Objects.isNull(floorObject) || floorObject.isEmpty()) {
                throw new IllegalArgumentException("楼层不存在");
            }

            groupExtra.put("floorId", floorId);
            groupExtra.put("floorName", DmpConstant.getIbmsxianshimingcheng(floorObject));

        }

        entity.setGroupExtra(groupExtra);

        // 执行更新
        if (isUpdate) {
            logicalGroupingMapper.updateById(entity);
        } else {
            // 新建时获取最大序列号
            entity.setSequence(
                    generateSequence(
                            groupCode,
                            projectId,
                            ibmsSceneCode,
                            ibmsClassCode,
                            parentLogicalGroupingId)
            );
            logicalGroupingMapper.insert(entity);
            logicalGroupingId = entity.getLogicalGroupingId();
        }

        // 保存关系
        logicalObjectServiceV2.saveBindLogicalObjects(
                groupCode, projectId, ibmsSceneCode, ibmsClassCode,
                logicalGroupingId, request.getObjIds());
    }

    private int generateSequence(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, String parentId) {
        return 1 + logicalGroupingMapper.selectObjs(
                Wrappers.<LogicalGrouping>query()
                        .select("coalesce(max(`sequence`), 0)")
                        .lambda()
                        .eq(LogicalGrouping::getGroupCode, groupCode)
                        .eq(LogicalGrouping::getProjectId, projectId)
                        .eq(LogicalGrouping::getIbmsSceneCode, ibmsSceneCode)
                        .eq(LogicalGrouping::getIbmsClassCode, ibmsClassCode)
                        .eq(LogicalGrouping::getParentId, parentId))
                .stream()
                .map(String::valueOf)
                .map(Integer::valueOf)
                .findFirst()
                .orElse(0);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteLogicalGrouping(DeleteLogicalGroupingRequest request) {
        String groupCode = request.getGroupCode();
        String projectId = request.getProjectId();
        String ibmsSceneCode = request.getIbmsSceneCode();
        String ibmsClassCode = request.getIbmsClassCode();
        String logicalGroupingId = request.getLogicalGroupingId();

        // 先处理逻辑对象数据
        logicalObjectServiceV2.removeBindLogicalObjects(
                groupCode,
                projectId,
                ibmsSceneCode,
                ibmsClassCode,
                logicalGroupingId
        );

        //
        // 删除
        logicalGroupingMapper.delete(
                Wrappers.<LogicalGrouping>lambdaQuery()
                        .eq(LogicalGrouping::getLogicalGroupingId, logicalGroupingId)
                        .or()
                        .eq(LogicalGrouping::getParentId, logicalGroupingId)
        );

    }
}
