package com.persagy.dmp.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.persagy.dmp.dto.EquipBelongModularDTO;
import com.persagy.dmp.dto.EquipLocationDTO;
import com.persagy.dmp.dto.LocationDTO;
import com.persagy.dmp.entity.LogicalObjectEquip;
import com.persagy.dmp.entity.LogicalObjectEquipCached;
import com.persagy.dmp.mapper.LogicalObjectEquipMapper;
import com.persagy.dmp.service.LocationService;
import com.persagy.dmp.service.LogicalObjectEquipCachedService;
import com.persagy.dmp.service.LogicalObjectEquipService;
import com.persagy.dmp.service.ObjectService;
import com.persagy.dmp.utils.DataUtils;
import lombok.SneakyThrows;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.stream.Collectors;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/13 11:06
 */
@Service
public class LogicalObjectEquipServiceImpl extends ServiceImpl<LogicalObjectEquipMapper, LogicalObjectEquip> implements LogicalObjectEquipService {

    private static final String LIGHTING_CLASS_CODE = "zhaominghuilu";

    // 清理locks保证线程同步, 换成分布式锁时可以删掉
    private final ReentrantReadWriteLock readWriteLock = new ReentrantReadWriteLock();

    // 多节点部署时再换成分布式锁
    private final ConcurrentHashMap<String, ReentrantLock> locks = new ConcurrentHashMap<>();

    @Autowired
    private ObjectService objectService;

    @Autowired
    private LocationService locationService;

    @Autowired
    private LogicalObjectEquipCachedService logicalObjectEquipCachedService;

    @Override
    public void loadIfAbsent(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        LogicalObjectEquipCached entity = logicalObjectEquipCachedService.getOne(
                groupCode, projectId, ibmsSceneCode, ibmsClassCode);

        if (Objects.nonNull(entity)) {
            return;
        }

        load(groupCode, projectId, ibmsSceneCode, ibmsClassCode, true);
    }

    @Override
    public void load(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, boolean waitFor) {
        ReentrantReadWriteLock.ReadLock readLock = readWriteLock.readLock();

        readLock.lock();
        ReentrantLock lock = getLock(groupCode, projectId, ibmsSceneCode, ibmsClassCode);

        if (!lock.tryLock()) {
            // 尝试获取锁失败, 但是已经拿到lock对象了, 这里只需要等待lock释放, 所以把readLock释放, 要不是获取writeLock会死锁
            readLock.unlock();
            // 如果有其他线程在加载, waitFor为true, 那等待其他线程加载完成, 否则直接退出
            if (waitFor) {
                try {
                    lock.lock();
                } finally {
                    lock.unlock();
                }

            }

            return;
        }

        try {
            // 加载数据
            loadInterval(
                    groupCode,
                    projectId,
                    ibmsSceneCode,
                    ibmsClassCode,
                    LIGHTING_CLASS_CODE.equals(ibmsClassCode));
        } finally {
            ReentrantReadWriteLock.WriteLock writeLock = readWriteLock.writeLock();

            try {
                readLock.unlock();
                writeLock.lock();

                removeLock(groupCode, projectId, ibmsSceneCode, ibmsClassCode);

                lock.unlock();
            } finally {
                writeLock.unlock();
            }
        }

    }

    private ReentrantLock getLock(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        return locks.computeIfAbsent(
                lockKey(groupCode, projectId, ibmsSceneCode, ibmsClassCode),
                k -> new ReentrantLock());
    }

    private void removeLock(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        locks.remove(lockKey(groupCode, projectId, ibmsSceneCode, ibmsClassCode));
    }

    private String lockKey(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        return String.join(":", groupCode, projectId, ibmsSceneCode, ibmsClassCode);
    }

    @SneakyThrows
    private void loadInterval(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, boolean isLoadLightInfo) {
        JSONArray allObjectArray = objectService.queryObjectList(groupCode, projectId, ibmsSceneCode, ibmsClassCode, null).getData(JSONArray.class);

        List<String> objIds = Lists.newLinkedList();
        Map<String, JSONObject> objectMap = Maps.newHashMap();
        for (Object o : allObjectArray) {
            JSONObject json = (JSONObject) o;
            String objType = json.getString("objType");

            if (!"equipment".equals(objType)) {
                continue;
            }


            String objId = json.getString("id");
            objectMap.put(objId, json);


            objIds.add(objId);
        }

        Map<String, EquipLocationDTO> equipLocationDTOMap = locationService.queryEquipLocationMap(groupCode, projectId, objIds);
        Map<String, EquipBelongModularDTO> equipBelongModularDTOMap = isLoadLightInfo ?
                locationService.queryEquipModularBox(groupCode, projectId, objIds) :
                Collections.emptyMap();

        // 查询出当前已经存在的数据
        Map<String, String> exists = ObjectUtils.defaultIfNull(
                list(
                        Wrappers.<LogicalObjectEquip>lambdaQuery()
                                .select(LogicalObjectEquip::getId, LogicalObjectEquip::getObjId)
                                .eq(LogicalObjectEquip::getGroupCode, groupCode)
                                .eq(LogicalObjectEquip::getProjectId, projectId)
                                .eq(LogicalObjectEquip::getIbmsSceneCode, ibmsSceneCode)
                                .eq(LogicalObjectEquip::getIbmsClassCode, ibmsClassCode)
                                .in(CollectionUtils.isNotEmpty(objIds), LogicalObjectEquip::getObjId, objIds)),
                Collections.<LogicalObjectEquip>emptyList())
                .stream()
                .collect(Collectors.toMap(LogicalObjectEquip::getObjId, LogicalObjectEquip::getId));


        List<LogicalObjectEquip> toSaveList = Lists.newLinkedList();
        List<LogicalObjectEquip> toUpdateList = Lists.newLinkedList();
        for (String objId : objIds) {
            JSONObject json = objectMap.get(objId);
            String objType = json.getString("objType");

            LogicalObjectEquip entity = new LogicalObjectEquip();
            entity.setGroupCode(groupCode);
            entity.setProjectId(projectId);
            entity.setIbmsSceneCode(ibmsSceneCode);
            entity.setIbmsClassCode(ibmsClassCode);
            entity.setObjId(objId);
            entity.setObjType(objType);
            entity.setValid(json.getInteger("valid"));


            // 设置位置信息
            LocationDTO locationDTO = json.toJavaObject(LocationDTO.class);
            EquipLocationDTO equipLocationDTO = equipLocationDTOMap.get(objId);

            if (Objects.isNull(equipLocationDTO)) {
                equipLocationDTO = new EquipLocationDTO();
            }

            BeanUtils.copyProperties(locationDTO, equipLocationDTO);
            entity.setObjName(equipLocationDTO.getIbmsxianshimingcheng());
            entity.setLocalId(equipLocationDTO.getIbmsxianshibianma());

            entity.setGroupExtra(JSONObject.parseObject(JSON.toJSONString(equipLocationDTO)));

            entity.setLocation(DataUtils.join(
                    "-",
                    equipLocationDTO.getBuildName(),
                    equipLocationDTO.getFloorName(),
                    equipLocationDTO.getSpaceName()));

            // 照明相关
            EquipBelongModularDTO equipBelongModularDTO = equipBelongModularDTOMap.get(objId);

            if (Objects.nonNull(equipBelongModularDTO)) {
                entity.setModularId(equipBelongModularDTO.getBelongModularId());
                entity.setModularName(equipBelongModularDTO.getBelongModularName());
                entity.setPowerBoxId(equipBelongModularDTO.getBelongPowerBoxId());
                entity.setPowerBoxName(equipBelongModularDTO.getBelongPowerBoxName());
                entity.setPowerWellsId(equipBelongModularDTO.getPowerWellsId());
                entity.setPowerWellsName(equipBelongModularDTO.getPowerWellsName());
            }

            String id = exists.get(objId);

            if (Objects.isNull(id)) {
                toSaveList.add(entity);
                continue;
            }

            entity.setId(id);
            toUpdateList.add(entity);
        }

        if (CollectionUtils.isNotEmpty(toSaveList)) {
            saveBatch(toSaveList, 200);
        }

        if (CollectionUtils.isNotEmpty(toUpdateList)) {
            updateBatchById(toUpdateList, 200);
        }

        if (CollectionUtils.isNotEmpty(objIds)) {
            // 移除已删除的数据
            remove(Wrappers.<LogicalObjectEquip>lambdaQuery()
                    .eq(LogicalObjectEquip::getGroupCode, groupCode)
                    .eq(LogicalObjectEquip::getProjectId, projectId)
                    .eq(LogicalObjectEquip::getIbmsSceneCode, ibmsSceneCode)
                    .eq(LogicalObjectEquip::getIbmsClassCode, ibmsClassCode)
                    .notIn(LogicalObjectEquip::getObjId, objIds)
            );
        }

        // 标记加载完成
        logicalObjectEquipCachedService.saveOrUpdate(groupCode, projectId, ibmsSceneCode, ibmsClassCode);
    }

    @Override
    public boolean tryLoadIfAbsentWithGroupings(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        List<LogicalObjectEquipCached> needLoads = logicalObjectEquipCachedService.listUnloadEquipCached(
                groupCode, projectId, ibmsSceneCode, ibmsClassCode);

        if (CollectionUtils.isEmpty(needLoads)) {
            return true;
        }

        // 异步加载数据
        CompletableFuture.runAsync(() -> {
            for (LogicalObjectEquipCached needLoad : needLoads) {
                this.load(
                        needLoad.getGroupCode(),
                        needLoad.getProjectId(),
                        needLoad.getIbmsSceneCode(),
                        needLoad.getIbmsClassCode(),
                        false
                );
            }
        });


        return false;
    }
}
