package com.persagy.dmp.service.impl;

import java.util.*;
import java.util.concurrent.*;

import com.persagy.dmp.cache.RedisCache;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.dmp.cache.ObjectCache;
import com.persagy.dmp.cache.RelationCache;
import com.persagy.dmp.common.PageResult;
import com.persagy.dmp.constant.DmpConstant;
import com.persagy.dmp.constant.RedisConstant;
import com.persagy.dmp.dao.InstanceDao;
import com.persagy.dmp.dto.ObjectInfoDTO;
import com.persagy.dmp.dto.RelationFromDTO;
import com.persagy.dmp.dto.RelationToDTO;
import com.persagy.dmp.enumeration.EnumGraphCode;
import com.persagy.dmp.service.ObjectService;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * @description: 对象接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:11
 * @version: V1.0
 **/
@Service
@Slf4j
public class ObjectServiceImpl implements ObjectService {

    @Autowired
    private InstanceDao instanceDao;

    @Autowired
    private ObjectCache objectCache;

    @Autowired
    private RelationCache relationCache;

    @Autowired
    private RedisCache redisCache;

    @Override
    public Boolean createObject(String groupCode, String projectId, @NonNull String classCode, JSONObject objectInfo) {
        return instanceDao.createObject(groupCode, projectId, classCode, objectInfo);
    }

    @Override
    public Boolean createObject(String groupCode, String projectId, JSONArray objectArr) {
        return instanceDao.createObject(groupCode, projectId, objectArr);
    }

    @Override
    public Boolean updateObject(String groupCode, String projectId, String objectId, JSONObject objectInfo) {
        return instanceDao.updateObject(groupCode, projectId, objectId, objectInfo);
    }

    @Override
    public Boolean updateObject(String groupCode, String projectId, JSONArray objectArr) {
        return instanceDao.updateObject(groupCode, projectId, objectArr);
    }

    @Override
    public Boolean deleteObject(String groupCode, String projectId, String objectId) {
        return instanceDao.deleteObject(groupCode, projectId, objectId);
    }

    @Override
    public Boolean deleteObject(String groupCode, String projectId, List<String> objectIdList) {
        return instanceDao.deleteObject(groupCode, projectId, objectIdList);
    }

    @Override
    public String queryClassCode(String groupCode, String projectId, String objId) throws Exception {
        Map<String, String> id2ClassCodeMap = queryClassCode(groupCode, projectId, Arrays.asList(objId));
        return id2ClassCodeMap.get(objId);
    }

    @Override
    public Map<String, String> queryClassCode(String groupCode, String projectId, List<String> objIds) throws Exception {
        Map<String, String> id2ClassCodeMap = new HashMap<>();
        if (CollectionUtils.isEmpty(objIds)) {
            return id2ClassCodeMap;
        }

        Map<String, String> objId2ClassCodeMapCache = objectCache.getClassCode(groupCode, projectId, objIds);
        if (objId2ClassCodeMapCache != null) {
            if (objId2ClassCodeMapCache.size() == objIds.size()) {
                return objId2ClassCodeMapCache;
            } else {
                //取差集
                objIds.removeAll(objId2ClassCodeMapCache.keySet());
            }
        }

        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("id", objIds);
        criteria.setWithColumns(new HashSet<>(Arrays.asList("name", "localId", "localName", "classCode", "bimId")));
        PageResult<JSONArray> pageResult = instanceDao.queryObjectList(groupCode, projectId, criteria);
        JSONArray objectArr = pageResult.getData(JSONArray.class);
        if (!CollectionUtils.isEmpty(objectArr)) {
            List<ObjectInfoDTO> objectInfoDTOList = objectArr.toJavaList(ObjectInfoDTO.class);
            if (!CollectionUtils.isEmpty(objectInfoDTOList)) {
                objectInfoDTOList.forEach(o -> {
                    id2ClassCodeMap.put(o.getId(), o.getClassCode());
                });
            }
        }

        //异步存入缓存
        CompletableFuture.runAsync(() -> {
            objectCache.putClassCode(groupCode, projectId, id2ClassCodeMap);
        });

        if (!CollectionUtils.isEmpty(objId2ClassCodeMapCache)) {
            //缓存取到位置并入结果集
            id2ClassCodeMap.putAll(objId2ClassCodeMapCache);
        }
        return id2ClassCodeMap;
    }

    @Override
    public JSONObject queryObjectInfo(String groupCode, String projectId, String objectId) throws Exception {

        //查询缓存
        JSONObject objectJson = objectCache.getObjectById(groupCode, projectId, objectId);
        if (objectJson != null) {
            return objectJson;
        }

        //查询数据中台
        objectJson = instanceDao.queryObjectInfo(groupCode, projectId, objectId);

        //异步存入缓存
        JSONObject finalObjectJson = objectJson;
        CompletableFuture.runAsync(() -> {
            if (finalObjectJson != null) {
                objectCache.putObject(groupCode, projectId, finalObjectJson);
            }
        });

        return objectJson;
    }
    
    @Override
    public JSONObject queryObjectInfoNoCache(String groupCode, String projectId, String objectId) throws Exception {
        return instanceDao.queryObjectInfo(groupCode, projectId, objectId);
    }
    
	@Override
	public JSONObject queryObjectInfo(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, String objectId) throws Exception {
		return this.instanceDao.queryObjectInfo(groupCode, projectId, ibmsSceneCode, ibmsClassCode, objectId);
	}

    @Override
    public PageResult<JSONArray> queryObjectList(String groupCode, String projectId, String classCode) throws Exception {
        return this.queryObjectList(groupCode, projectId, classCode, null);
    }

    @Override
    public PageResult<JSONArray> queryObjectList(String groupCode, String projectId, String classCode, List<String> withColumns) throws Exception {
        PageResult<JSONArray> pageResult = instanceDao.queryObjectList(groupCode, projectId, classCode, withColumns);

        //异步存入缓存
        CompletableFuture.runAsync(() -> {
            if (pageResult != null && !CollectionUtils.isEmpty(pageResult.getData())) {
                objectCache.putObject(groupCode, projectId, classCode, pageResult.getData());
            }
        });
        return pageResult;
    }


    @Override
    public PageResult<JSONArray> queryObjectList(String groupCode, String projectId, String ibmsSceneCode,String ibmsClassCode, List<String> withColumns) throws Exception {
        return instanceDao.queryObjectList(groupCode, projectId, ibmsSceneCode,ibmsClassCode, withColumns);
    }

    @Override
    public JSONArray queryObjectList(String groupCode, String projectId, List<String> objectIdList) throws Exception {

        if (CollectionUtils.isEmpty(objectIdList)) {
            return new JSONArray();
        }

        int threadCounts = objectIdList.size() / 100;
        if (threadCounts == 0) {
            threadCounts = 1;
        } else if (threadCounts > 32) {
            threadCounts = 32;
        }

        ExecutorService exec = Executors.newFixedThreadPool(threadCounts);
        List<Callable<JSONArray>> callList = new ArrayList<>();

        //平均分割List
        int len = objectIdList.size() / threadCounts;
        //List中的数量没有线程数多（很少存在）
        if (len == 0) {
            //采用一个线程处理List中的一个元素
            threadCounts = objectIdList.size();
            //重新平均分割List
            len = objectIdList.size() / threadCounts;
        }

        for (int i = 0; i < threadCounts; i++) {
            final List<String> subList;
            if (i == threadCounts - 1) {
                subList = objectIdList.subList(i * len, objectIdList.size());
            } else {
                subList = objectIdList.subList(i * len, Math.min(len * (i + 1), objectIdList.size()));
            }
            //采用匿名内部类实现
            callList.add(() -> ObjectServiceImpl.this.querySubObjectList(groupCode, projectId, subList));
        }

        List<Future<JSONArray>> futureList = exec.invokeAll(callList);
        JSONArray resultArr = new JSONArray();
        for (Future<JSONArray> future : futureList) {
            JSONArray subResultArr = future.get();
            if (!CollectionUtils.isEmpty(subResultArr)) {
                resultArr.addAll(subResultArr);
            }
        }
        exec.shutdown();

        return resultArr;
    }

    private JSONArray querySubObjectList(String groupCode, String projectId, List<String> objectIdList) throws Exception {
        if (CollectionUtils.isEmpty(objectIdList)) {
            return new JSONArray();
        }
        //查询缓存
//        JSONArray objArrayCache = objectCache.getObjectByIds(groupCode, projectId, objectIdList);
//        if (!CollectionUtils.isEmpty(objArrayCache)) {
//            if (objArrayCache.size() == objectIdList.size()) {
//                return objArrayCache;
//            } else {
//                List<String> objIds = objArrayCache.stream().map(o -> ((JSONObject) o).getString("id")).collect(Collectors.toList());
//                //取差集
//                objectIdList.removeAll(objIds);
//            }
//        }

        //查询对象列表
        JSONArray objArray = instanceDao.queryObjectList(groupCode, projectId, objectIdList);

        //异步存入缓存
        CompletableFuture.runAsync(() -> {
            if (!CollectionUtils.isEmpty(objArray)) {
                try {
                	objectCache.putObject(groupCode, projectId, objArray);
				} catch (Exception e) {
				}
            }
        });

//        if (!CollectionUtils.isEmpty(objArray)) {
//            objArrayCache.addAll(objArray);
//        }

        return objArray;
    }

    @Override
    public Map<String, ObjectInfoDTO> queryObjectMap(String groupCode, String projectId, List<String> objectIdList) throws Exception {
        Map<String, ObjectInfoDTO> id2ObjectInfoMap = new HashMap<>();
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("id", objectIdList);
        criteria.setWithColumns(new HashSet<>(Arrays.asList("name", "localId", "localName", "classCode", "bimId", "ibmsxianshibianma", "ibmsxianshimingcheng", "wD_huiyunbendibianma","wD_huiyunbendimingcheng")));
        PageResult<JSONArray> pageResult = instanceDao.queryObjectList(groupCode, projectId, criteria);
        JSONArray objectArr = pageResult.getData(JSONArray.class);
        if (!CollectionUtils.isEmpty(objectArr)) {
            List<ObjectInfoDTO> objectInfoDTOList = objectArr.toJavaList(ObjectInfoDTO.class);
            if (!CollectionUtils.isEmpty(objectInfoDTOList)) {
                objectInfoDTOList.forEach(o -> {
                    id2ObjectInfoMap.put(o.getId(), o);
                });
            }
        }
        return id2ObjectInfoMap;
    }

    @Override
    public PageResult<JSONArray> queryObjectList(String groupCode, String projectId, JacksonCriteria criteria) throws Exception {
        return instanceDao.queryObjectList(groupCode, projectId, criteria);
    }

    @Override
    public PageResult<JSONArray> queryObjectList(String groupCode, String projectId, JSONObject criteria) throws Exception {
        return instanceDao.queryObjectList(groupCode, projectId, criteria);
    }

    @Override
    public PageResult<JSONArray> queryObjectList(String groupCode, String projectId, JacksonCriteria criteria, RelationFromDTO relationFromDTO, RelationToDTO relationToDTO) throws Exception {
        return instanceDao.queryObjectList(groupCode, projectId, criteria, relationFromDTO, relationToDTO);
    }

    @Override
    public List<ObjectInfoDTO> queryEquipByCategory(String groupCode, String projectId, String category) throws Exception {
        JSONArray objectList = queryObjectList(groupCode, projectId, category).getData();
        if (CollectionUtils.isEmpty(objectList)) {
            return new ArrayList<>();
        }
        return objectList.toJavaList(ObjectInfoDTO.class);
    }

    @Override
    public JSONArray queryBuildArrByProjectId(String groupCode, String projectId, List<String> withColumns) throws Exception {
        String key = redisCache.joinKeys(
                groupCode,
                projectId,
                redisCache.md5(withColumns)
        );

        JSONArray objectList = redisCache.get(RedisConstant.PROJECT_BUILD_CACHE, key, JSONArray.class);

        if (Objects.isNull(objectList)) {
            objectList = instanceDao.queryObjectList(groupCode, projectId, DmpConstant.ObjectType.BUILDING, withColumns).getData();

            if (CollectionUtils.isEmpty(objectList)) {
                objectList = new JSONArray();
            }

            redisCache.put(RedisConstant.PROJECT_BUILD_CACHE, key, objectList, 60, TimeUnit.MINUTES);
        }
        return objectList;
    }

    @Override
    public List<ObjectInfoDTO> queryBuildByProjectId(String groupCode, String projectId) throws Exception {
        JSONArray objectList = queryBuildArrByProjectId(groupCode, projectId, Arrays.asList("classCode"));
        return objectList.toJavaList(ObjectInfoDTO.class);
    }

    @Override
    public JSONArray queryFloorArrByBuildId(String groupCode, String projectId, String buildId, List<String> withColumns) throws Exception {
        String key = redisCache.joinKeys(
                groupCode,
                projectId,
                buildId,
                redisCache.md5(withColumns)
        );

        JSONArray objectList = redisCache.get(RedisConstant.BUILD_FLOOR_CACHE, key, JSONArray.class);

        if (Objects.isNull(objectList)) {
             objectList = instanceDao.queryRelationObjectByFromId(
                    groupCode, projectId, EnumGraphCode.ArchSubset.getCode(),
                    DmpConstant.RelationType.ARCH_SUBSET_BUILDING_FLOOR, buildId,
                    null, withColumns);

            if (CollectionUtils.isEmpty(objectList)) {
                objectList = new JSONArray();
            }

            redisCache.put(RedisConstant.BUILD_FLOOR_CACHE, key, objectList, 60, TimeUnit.MINUTES);
        }

        return objectList;
    }

    @Override
    public List<ObjectInfoDTO> queryFloorByBuildId(String groupCode, String projectId, String buildId) throws Exception {
        JSONArray objectList = queryFloorArrByBuildId(groupCode, projectId, buildId, null);
        return objectList.toJavaList(ObjectInfoDTO.class);
    }

    @Override
    public JSONArray querySystemArrByBuildId(String groupCode, String projectId, String buildId, List<String> withColumns) throws Exception {
        JSONArray objectList = instanceDao.queryRelationObjectByToId(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Sy2Bd", null, buildId, withColumns);
        if (CollectionUtils.isEmpty(objectList)) {
            return new JSONArray();
        }
        return objectList;
    }
    
    @Override
    public JSONArray querySystemArrByFloorId(String groupCode, String projectId, String floorId, List<String> withColumns) throws Exception {
        JSONArray objectList = instanceDao.queryRelationObjectByToId(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Sy2Fl", null, floorId, withColumns);
        if (CollectionUtils.isEmpty(objectList)) {
            return new JSONArray();
        }
        return objectList;
    }
    
    @Override
    public JSONArray querySystemArrBySpaceId(String groupCode, String projectId, String spaceId, List<String> withColumns) throws Exception {
        JSONArray objectList = instanceDao.queryRelationObjectByToId(groupCode, projectId, EnumGraphCode.MechForArch.getCode(), "Sy2Sp", null, spaceId, withColumns);
        if (CollectionUtils.isEmpty(objectList)) {
            return new JSONArray();
        }
        return objectList;
    }
    
    @Override
    public JSONArray querySpaceArrByFloorId(String groupCode, String projectId, String floorId, String classCode, String roomFuncType, List<String> withColumns) throws Exception {
        JSONObject objectCriteria = new JSONObject();
        if (!StringUtils.isEmpty(classCode)) {
            objectCriteria.put("classCode", classCode);
        }
        if (!StringUtils.isEmpty(roomFuncType)) {
            //空间功能类型
            objectCriteria.put("roomFuncType", roomFuncType);
        }
        //查询楼层下的特定业务分区特定功能类型的空间
        JSONArray objectList = instanceDao.queryRelationObject(groupCode, projectId, EnumGraphCode.ArchSubset.getCode(), "Fl2Sp", Arrays.asList(floorId), null, objectCriteria, withColumns);
        if (CollectionUtils.isEmpty(objectList)) {
            return new JSONArray();
        }
        return objectList;
    }

    @Override
    public List<ObjectInfoDTO> querySpaceByFloorId(String groupCode, String projectId, String floorId, String classCode, String roomFuncType) throws Exception {
        JSONArray objectList = querySpaceArrByFloorId(groupCode, projectId, floorId, classCode, roomFuncType, null);
        return objectList.toJavaList(ObjectInfoDTO.class);
    }

    @Override
    public JSONArray queryEquipArrBySpaceId(String groupCode, String projectId, String spaceId, String category, List<String> withColumns) throws Exception {
        //查询空间下的设备
        JSONArray objectList = instanceDao.queryRelationObjectByToId(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Eq2Sp", StringUtils.isNotEmpty(category) ? Arrays.asList(category) : null, spaceId, withColumns);
        return objectList;
    }

    @Override
    public List<ObjectInfoDTO> queryEquipBySpaceId(String groupCode, String projectId, String spaceId, String category) throws Exception {
        JSONArray objectList = queryEquipArrBySpaceId(groupCode, projectId, spaceId, category, null);
        return objectList.toJavaList(ObjectInfoDTO.class);
    }

    @Override
    public JSONArray queryEquipArrByFloorId(String groupCode, String projectId, String floorId, String category, List<String> withColumns) throws Exception {
        //查询楼层下的设备
        JSONArray objectList = instanceDao.queryRelationObjectByToId(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Eq2Fl", StringUtils.isNotEmpty(category) ? Arrays.asList(category) : null, floorId, withColumns);
        return objectList;
    }

    @Override
    public List<ObjectInfoDTO> queryEquipByFloorId(String groupCode, String projectId, String floorId, String category) throws Exception {
        JSONArray objectList = queryEquipArrByFloorId(groupCode, projectId, floorId, category, null);
        return objectList.toJavaList(ObjectInfoDTO.class);
    }

    @Override
    public JSONArray queryEquipArrBySystemId(String groupCode, String projectId, String systemId, List<String> classCodeList, List<String> withColumns) throws Exception {
        JSONArray objectList = instanceDao.queryRelationObjectByFromId(groupCode, projectId, EnumGraphCode.MechSubset.getCode(), "Sy2Eq", systemId, classCodeList, withColumns);
        if (CollectionUtils.isEmpty(objectList)) {
            return new JSONArray();
        }
        return objectList;
    }
    
    @Override
    public JSONArray queryEquipArrBySystemId(String groupCode, String projectId, String systemId, String ibmsSceneCode, String ibmsClassCode, List<String> withColumns) throws Exception {
        JSONArray objectList = instanceDao.queryRelationObjectByFromId(groupCode, projectId, EnumGraphCode.MechSubset.getCode(), "Sy2Eq", systemId, ibmsSceneCode, ibmsClassCode, withColumns);
        if (CollectionUtils.isEmpty(objectList)) {
            return new JSONArray();
        }
        return objectList;
    }

    @Override
    public List<ObjectInfoDTO> queryEquipBySystemId(String groupCode, String projectId, String systemId, List<String> classCodeList) throws Exception {
        JSONArray objectList = queryEquipArrBySystemId(groupCode, projectId, systemId, classCodeList, null);
        return objectList.toJavaList(ObjectInfoDTO.class);
    }

    @Override
    public JSONArray queryEquipArrByBuildId(String groupCode, String projectId, String buildId, String category, List<String> withColumns) throws Exception {
        JSONArray objArr = relationCache.getRelation(RedisConstant.EQUIP_IN_BUILD_PREFIX, groupCode, projectId, buildId, category);
        if (!CollectionUtils.isEmpty(objArr)) {
            return objArr;
        }
        JSONArray objectList = instanceDao.queryRelationObjectByToId(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Eq2Bd", StringUtils.isNotEmpty(category) ? Arrays.asList(category) : null, buildId, withColumns);
        if (!CollectionUtils.isEmpty(objectList)) {
            //异步存入缓存
            CompletableFuture.runAsync(() -> {
                relationCache.putRelation(RedisConstant.EQUIP_IN_BUILD_PREFIX, groupCode, projectId, buildId, category, objectList);
            });

            return objectList;
        }
        return new JSONArray();
    }

    /**
     * @description: 校验 对象 信息值 是否重复
     * @param: projectId 项目id
     * @param: objectId    对象id
     * @param: infoCode    信息点
     * @param: value   信息点值
     * @return: java.lang.Boolean
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/4 14:46
     * @version: V1.0
     */
    @Override
    public Boolean verifyIsRepeat(String groupCode, @NonNull String projectId, String objectId, @NonNull String infoCode, @NonNull Object value) throws Exception {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add(infoCode).in(Arrays.asList(value));
        criteria.add("grouping",2);
        criteria.add("valid",1);
        JSONArray objectArr = instanceDao.queryObjectList(groupCode, projectId, criteria).getData();
        if (!CollectionUtils.isEmpty(objectArr)) {
            if (StringUtils.isEmpty(objectId)) {
                //新建
                return true;
            } else {
                //编辑
                for (int i = 0; i < objectArr.size(); i++) {
                    JSONObject jsonObject = objectArr.getJSONObject(i);
                    String id = jsonObject.getString("id");
                    if (objectId.equals(id)) {
                        return false;
                    }
                }
                return true;
            }
        }
        return false;
    }

    @Override
    public Map<String, String> queryFloorSequenceID(@NonNull String groupCode, @NonNull String projectId, Set<String> floorIds) {
        Map<String,String> floorSequenceIDMap = new HashMap<>();
        JSONObject jsonCriteria = new JSONObject();
        JSONObject jsonObjectItem = new JSONObject();
        jsonObjectItem.put("id",floorIds);
        jsonObjectItem.put("valid",1);
        jsonCriteria.put("criteria",jsonObjectItem);
        try {
            JSONArray objectArr = instanceDao.queryObjectList(groupCode, projectId, jsonCriteria).getData();
            for (Object obj:objectArr
                 ) {
                floorSequenceIDMap.put(((JSONObject)obj).getString("id"),((JSONObject)obj).getString("floorSequenceID"));
            }

        }catch (Exception e){
           log.error(e.getMessage());
        }
        return floorSequenceIDMap;
    }

	@Override
	public JSONObject queryRelationObjectByToId(String groupCode, String projectId, String graphCode, String relCode, String toId) throws Exception {
		JSONArray array = this.instanceDao.queryRelationObjectByToIds(groupCode, projectId, graphCode, relCode, null, Lists.newArrayList(toId), null);
		return (array == null || array.size() == 0) ? null : array.getJSONObject(0);
	}

	@Override
	public JSONObject queryRelationObjectByFromId(String groupCode, String projectId, String graphCode, String relCode, String fromId) throws Exception {
		JSONArray array = this.instanceDao.queryRelationObjectByFromId(groupCode, projectId, graphCode, relCode, fromId, null, null);
		return (array == null || array.size() == 0) ? null : array.getJSONObject(0);
	}

}
