package com.persagy.dmp.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import com.persagy.dmp.dto.CurrentDataDTO;
import com.persagy.dmp.dto.DataQueryDTO;
import com.persagy.dmp.dto.DataTimeDTO;
import com.persagy.dmp.dto.HistoryDataDTO;
import com.persagy.dmp.service.CurrentDataService;
import com.persagy.dmp.service.HistoryDataService;
import com.persagy.dmp.service.ObjectStatisticsService;

import cn.hutool.core.util.StrUtil;

/**
 * @description: 统计相关实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/9 20:43
 * @version: V1.0
 **/
@Service
public class ObjectStatisticsServiceImpl implements ObjectStatisticsService {

    @Autowired
    private CurrentDataService currentDataService;

    @Autowired
    private HistoryDataService historyDataService;

    @Override
    public BigDecimal queryInfoCodeAverage(String groupCode, String projectId, List<String> objectIds, String infoCode) throws Exception {
        if (CollectionUtils.isEmpty(objectIds) || StringUtils.isBlank(infoCode)) {
            return new BigDecimal(0);
        }
        List<DataQueryDTO> objectIdInfoCodeList = objectIds.stream()
                .flatMap(objectId -> Stream.of(new DataQueryDTO(objectId, infoCode))).collect(Collectors.toList());
        //获取采集值
        List<CurrentDataDTO> currentDataDTOList = currentDataService.queryCurrentDataList(groupCode, projectId, objectIdInfoCodeList);

        return queryInfoCodeAverage(currentDataDTOList);
    }

    @Override
    public BigDecimal queryInfoCodeAverage(List<CurrentDataDTO> currentDataDTOList) {
        if (CollectionUtils.isEmpty(currentDataDTOList)) {
            return new BigDecimal(0);
        }

        //过滤掉没有采集值的数据
        List<BigDecimal> dataList = currentDataDTOList.stream().filter(o -> o.getData() != null && !StrUtil.isBlankOrUndefined(o.getTime())).map(CurrentDataDTO::getData).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(dataList)) {
            //求平均值
            BigDecimal sum = dataList.stream().reduce(BigDecimal.ZERO, BigDecimal::add);
            return sum.divide(new BigDecimal(dataList.size()), 2);
        }
        return new BigDecimal(0);
    }


    @Override
    public List<DataTimeDTO> queryHistoryInfoCodeAverage(String groupCode, String projectId, List<String> objectIds, String infoCode, String startTime, String endTime, String period) throws Exception {
        if (CollectionUtils.isEmpty(objectIds) || StringUtils.isBlank(infoCode)) {
            return new ArrayList<>();
        }
        List<DataQueryDTO> objectIdInfoCodeList = objectIds.stream()
                .flatMap(objectId -> Stream.of(new DataQueryDTO(objectId, infoCode))).collect(Collectors.toList());

        //获取历史数据
        List<HistoryDataDTO> historyDataDTOList = historyDataService.queryHistoryDataList(groupCode, projectId, objectIdInfoCodeList, startTime, endTime, period);
        if (CollectionUtils.isEmpty(historyDataDTOList)) {
            return new ArrayList<>();
        }

        //把各个对象数据归纳到一块
        List<DataTimeDTO> dataTimeDTOList = new ArrayList<>();
        for (HistoryDataDTO historyDataDTO : historyDataDTOList) {
            if (!CollectionUtils.isEmpty(historyDataDTO.getData())) {
                dataTimeDTOList.addAll(historyDataDTO.getData());
            }
        }

        //数据按照时间分组
        Map<String, List<DataTimeDTO>> map = dataTimeDTOList.stream().collect(Collectors.groupingBy(o -> o.getTime()));
        List<DataTimeDTO> result = new ArrayList<>();
        for (Map.Entry<String, List<DataTimeDTO>> entry : map.entrySet()) {
            String time = entry.getKey();
            List<DataTimeDTO> list = entry.getValue();
            //过滤掉没有采集值的数据
            List<BigDecimal> dataList = list.stream().filter(o -> o.getData() != null && !StrUtil.isBlankOrUndefined(o.getTime())).map(DataTimeDTO::getData).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(dataList)) {
                BigDecimal sum = dataList.stream().reduce(BigDecimal.ZERO, BigDecimal::add);
                BigDecimal average = sum.divide(new BigDecimal(dataList.size()), 2);
                result.add(DataTimeDTO.builder().time(time).data(average).build());
            }
        }

        //按照时间排序
        result = result.stream().sorted(Comparator.comparing(DataTimeDTO::getTime)).collect(Collectors.toList());

        return result;
    }

    @Override
    public Long queryInfoCodeStatistics(String groupCode, String projectId, List<String> objectIds, String infoCode, BigDecimal data) throws Exception {
        if (CollectionUtils.isEmpty(objectIds) || StringUtils.isBlank(infoCode) || data == null) {
            return 0L;
        }

        List<DataQueryDTO> objectIdInfoCodeList = objectIds.stream()
                .flatMap(objectId -> Stream.of(new DataQueryDTO(objectId, infoCode))).collect(Collectors.toList());
        //获取采集值
        List<CurrentDataDTO> currentDataDTOList = currentDataService.queryCurrentDataList(groupCode, projectId, objectIdInfoCodeList);
        if (CollectionUtils.isEmpty(currentDataDTOList)) {
            return 0L;
        }
        //过滤掉无效数据，没绑点的、没日期的
        currentDataDTOList = currentDataDTOList.stream().filter(o -> o.getData() != null && !StrUtil.isBlankOrUndefined(o.getTime())).collect(Collectors.toList());
        //统计
        return currentDataDTOList.stream().filter(o -> data.compareTo(o.getData()) == 0).count();
    }
}
