package com.persagy.dmp.utils;

import org.apache.commons.lang3.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.Principal;

/**
 * 请求处理工具类
 * 
 * @description 
 * @author zhangqiankun
 * @since 2020年11月2日:	下午2:24:13
 */
public class RequestUtil {
	
    private static final String X_FORWARDED_FOR = "X-forwarded-for";
    
    private static final String PROXY_CLIENT_IP = "Proxy-Client-IP";
    
    private static final String WL_PROXY_CLIENT_IP = "WL-Proxy-Client-IP";
    
    private static final String HTTP_CLIENT_IP = "HTTP_CLIENT_IP";
    
    private static final String UNKNOWN = "unknown";
    
    private static final String HTTP_X_FORWARDED_FOR = "HTTP_X_FORWARDED_FOR";
    
    private static final String COMMA = ",";

    
    /**
     * Description:获取ip地址
     * @return String
     * @author lijie
     * @since 2018年10月22日: 下午4:57:26
     * Update By lijie 2018年10月22日: 下午4:57:26
     */
    public static String getIpAddress() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
            if (ip.equals("127.0.0.1")) {
                // 根据网卡取本机配置的IP
                InetAddress inet = null;
                try {
                    inet = InetAddress.getLocalHost();
                } catch (UnknownHostException e) {
                    e.printStackTrace();
                }
                ip = inet.getHostAddress();
            }
        }
        // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        if (ip != null && ip.length() > 15) {
            if (ip.indexOf(",") > 0) {
                ip = ip.substring(0, ip.indexOf(","));
            }
        }
        return ip;
    }
    
    
    /**
     * 从 request 中获取请求来源真实地址
     *
     * @param request 当前请求
     * @return 真实来源地址
     */
    public static String getIpAddress(HttpServletRequest request) {
        if (request == null) {
            return "";
        }
        String ip = request.getHeader(X_FORWARDED_FOR);
        if (ip == null || ip.isEmpty() || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(PROXY_CLIENT_IP);
        }
        if (ip == null || ip.isEmpty() || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(WL_PROXY_CLIENT_IP);
        }
        if (ip == null || ip.isEmpty() || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(HTTP_CLIENT_IP);
        }
        if (ip == null || ip.isEmpty() || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(HTTP_X_FORWARDED_FOR);
        }
        if (ip == null || ip.isEmpty() || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        if (ip != null && ip.contains(COMMA)) {
            // 对于通过多个代理的情况，取第一个非 unknown 的有效IP字符串
            String[] ips = ip.split(COMMA);
            for (String s : ips) {
                if (!UNKNOWN.equalsIgnoreCase(s)) {
                    ip = s;
                    break;
                }
            }
        }
        return ip;
    }

    /**
     * 获取完整的请求路径, 包含 queryString
     *
     * @param request 当前请求
     * @return 完整的请求路径
     */
    public static String getFullRequestUrl(HttpServletRequest request) {
        if (request == null) {
            return "";
        }
        if (StringUtils.isBlank(request.getQueryString())) {
            return request.getRequestURL().toString();
        }
        return request.getRequestURL().append('?').append(request.getQueryString()).toString();
    }

    /**
     * 获取完整的请求路径, 包含 queryString
     *
     * @param requestUrl  请求url
     * @param queryString 请求参数
     * @return 完整请求路径
     */
    public static String getFullRequestUrl(String requestUrl, String queryString) {
        if (StringUtils.isBlank(queryString) && StringUtils.isBlank(requestUrl)) {
            return "";
        }
        if (StringUtils.isBlank(queryString)) {
            return requestUrl;
        }
        return requestUrl + "?" + queryString;
    }


    /**
     * 从 request 中获取当前用户凭证
     *
     * @param request 当前请求
     * @return 用户凭证
     */
    public static String getUserPrincipal(HttpServletRequest request) {
        if (request == null) {
            return "";
        }
        
        Principal principal = request.getUserPrincipal();
        if (principal != null && StringUtils.isNotBlank(principal.getName())) {
            return principal.getName();
        }
        
        return "";
    }
}
